@@ -48,7 +48,7 @@ pub struct SliceSegments<'a> {
 }
 
 impl <'a> message::ReaderSegments for SliceSegments<'a> {
-    fn get_segment<'b>(&'b self, id: u32) -> Option<&'b [u8]> {
+    fn get_segment(&self, id: u32) -> Option<&[u8]> {
         if id < self.segment_indices.len() as u32 {
             let (a, b) = self.segment_indices[id as usize];
             Some(&self.words[(a * BYTES_PER_WORD)..(b * BYTES_PER_WORD)])
@@ -153,7 +153,7 @@ impl<T: Deref<Target = [u8]>> BufferSegments<T> {
 }
 
 impl <T: Deref<Target = [u8]>> message::ReaderSegments for BufferSegments<T> {
-    fn get_segment<'b>(&'b self, id: u32) -> Option<&'b [u8]> {
+    fn get_segment(&self, id: u32) -> Option<&[u8]> {
         if id < self.segment_indices.len() as u32 {
             let (a, b) = self.segment_indices[id as usize];
             Some(&self.buffer[(self.segment_table_bytes_len + a * BYTES_PER_WORD)..(self.segment_table_bytes_len + b * BYTES_PER_WORD)])
@@ -194,7 +194,7 @@ impl core::ops::DerefMut for OwnedSegments {
 }
 
 impl crate::message::ReaderSegments for OwnedSegments {
-    fn get_segment<'a>(&'a self, id: u32) -> Option<&'a [u8]> {
+    fn get_segment(&self, id: u32) -> Option<&[u8]> {
         if id < self.segment_indices.len() as u32 {
             let (a, b) = self.segment_indices[id as usize];
             Some(&self[(a * BYTES_PER_WORD)..(b * BYTES_PER_WORD)])
@@ -382,18 +382,14 @@ fn flatten_segments<R: message::ReaderSegments + ?Sized>(segments: &R) -> Vec<u8
     let segment_count = segments.len();
     let table_size = segment_count / 2 + 1;
     let mut result = Vec::with_capacity(word_count);
-    for _ in 0..(table_size * BYTES_PER_WORD) {
-        result.push(0);
-    }
+    result.resize(table_size * BYTES_PER_WORD, 0);
     {
         let mut bytes = &mut result[..];
         write_segment_table_internal(&mut bytes, segments).expect("Failed to write segment table.");
     }
     for i in 0..segment_count {
         let segment = segments.get_segment(i as u32).unwrap();
-        for idx in 0..segment.len() {
-            result.push(segment[idx]);
-        }
+        result.extend(segment);
     }
     result
 }
@@ -442,7 +438,7 @@ where W: Write, R: message::ReaderSegments + ?Sized {
                     &((segments.get_segment(idx as u32).unwrap().len() / BYTES_PER_WORD) as u32).to_le_bytes());
             }
             if segment_count == 2 {
-                for idx in 4..8 { buf[idx] = 0 }
+                for b in &mut buf[4..8] { *b = 0 }
             }
             write.write_all(&buf)?;
         } else {
@@ -452,7 +448,8 @@ where W: Write, R: message::ReaderSegments + ?Sized {
                     &((segments.get_segment(idx as u32).unwrap().len() / BYTES_PER_WORD) as u32).to_le_bytes());
             }
             if segment_count % 2 == 0 {
-                for idx in (buf.len() - 4)..(buf.len()) { buf[idx] = 0 }
+                let start_idx = buf.len() - 4;
+                for b in &mut buf[start_idx ..] { *b = 0 }
             }
             write.write_all(&buf)?;
         }
@@ -509,7 +506,7 @@ pub mod test {
                 read_segment_table, write_segment_table, write_segments};
 
     /// Writes segments as if they were a Capnproto message.
-    pub fn write_message_segments<W>(write: &mut W, segments: &Vec<Vec<crate::Word>>) where W: Write {
+    pub fn write_message_segments<W>(write: &mut W, segments: &[Vec<crate::Word>]) where W: Write {
         let borrowed_segments: &[&[u8]] = &segments.iter()
                                                    .map(|segment| crate::Word::words_to_bytes(&segment[..]))
                                                    .collect::<Vec<_>>()[..];
@@ -694,10 +691,9 @@ pub mod test {
         buf.clear();
     }
 
-    #[test]
     #[cfg_attr(miri, ignore)] // miri takes a long time with quickcheck
-    fn check_round_trip() {
-        fn round_trip(segments: Vec<Vec<crate::Word>>) -> TestResult {
+    quickcheck! {
+        fn test_round_trip(segments: Vec<Vec<crate::Word>>) -> TestResult {
             if segments.len() == 0 { return TestResult::discard(); }
             let mut buf: Vec<u8> = Vec::new();
 
@@ -710,16 +706,10 @@ pub mod test {
             }))
         }
 
-        quickcheck(round_trip as fn(Vec<Vec<crate::Word>>) -> TestResult);
-    }
-
-    #[test]
-    #[cfg_attr(miri, ignore)] // miri takes a long time with quickcheck
-    fn check_round_trip_slice_segments() {
-        fn round_trip(segments: Vec<Vec<crate::Word>>) -> TestResult {
+        fn test_round_trip_slice_segments(segments: Vec<Vec<crate::Word>>) -> TestResult {
             if segments.len() == 0 { return TestResult::discard(); }
             let borrowed_segments: &[&[u8]] = &segments.iter()
-                                                       .map(|segment| crate::Word::words_to_bytes(&segment[..]))
+                .map(|segment| crate::Word::words_to_bytes(&segment[..]))
                 .collect::<Vec<_>>()[..];
             let words = flatten_segments(borrowed_segments);
             let mut word_slice = &words[..];
@@ -731,8 +721,6 @@ pub mod test {
                 crate::Word::words_to_bytes(&segment[..]) == result_segments.get_segment(i as u32).unwrap()
             }))
         }
-
-        quickcheck(round_trip as fn(Vec<Vec<crate::Word>>) -> TestResult);
     }
 
     #[test]
@@ -752,9 +740,9 @@ pub mod test {
         let message = read_message_from_flat_slice(&mut byte_slice, message::ReaderOptions::new()).unwrap();
         assert_eq!(byte_slice, extra_bytes);
         let result_segments = message.into_segments();
-        for idx in 0..segments.len() {
+        for (idx, segment) in segments.iter().enumerate() {
             assert_eq!(
-                segments[idx],
+                *segment,
                 result_segments.get_segment(idx as u32).expect("segment should exist"));
         }
     }
