@@ -15,8 +15,9 @@ const U32_LEN_IN_BYTES: usize = core::mem::size_of::<u32>();
 /// Segments read from a buffer, useful for when you have the message in a buffer and don't want the
 /// extra copy of `read_message`.
 ///
-/// `NoAllocSliceSegments` is similar to [`SliceSegments`] but optimized for low memory embedded
-/// environment. It does not do heap allocations and consumes only one fat pointer worth of memory.
+/// `NoAllocSliceSegments` is similar to [`crate::serialize::SliceSegments`] but optimized for
+/// low memory embedded environment. It does not do heap allocations and consumes only one fat
+/// pointer worth of memory.
 ///
 /// # Performance considerations
 ///
@@ -121,13 +122,13 @@ impl<'b> NoAllocSliceSegments<'b> {
                 segment: &message[expected_data_offset..],
             })
         } else {
-            Ok(Self::MultipleSegments { message: message })
+            Ok(Self::MultipleSegments { message })
         }
     }
 }
 
 impl<'b> ReaderSegments for NoAllocSliceSegments<'b> {
-    fn get_segment<'a>(&'a self, idx: u32) -> Option<&'a [u8]> {
+    fn get_segment(&self, idx: u32) -> Option<&[u8]> {
         // panic safety: we are doing a lot of `unwrap` here. We assume that underlying message slice
         // holds valid capnp message - we already verified slice in NoAllocSliceSegments::try_new,
         // so these unwraps are not expected to panic unless we have bug in the code.
@@ -198,12 +199,12 @@ fn verify_alignment(ptr: *const u8) -> Result<()> {
     if ptr % BYTES_PER_WORD == 0 {
         Ok(())
     } else {
-        return Err(Error::failed(
+        Err(Error::failed(
             "Message was not aligned by 8 bytes boundary. \
             Either ensure that message is properly aligned or compile \
             `capnp` crate with \"unaligned\" feature enabled."
                 .to_string(),
-        ));
+        ))
     }
 }
 
@@ -316,10 +317,13 @@ fn calculate_data_offset(segments_count: usize) -> Option<usize> {
 
 #[cfg(test)]
 mod tests {
+    use quickcheck::{quickcheck, TestResult};
+
     use crate::{
         message::{ReaderOptions, ReaderSegments},
         serialize::{self, no_alloc_slice_segments::calculate_data_offset},
         OutputSegments,
+        Word,
     };
 
     use super::{
@@ -329,31 +333,6 @@ mod tests {
 
     use alloc::vec::Vec;
 
-    // Perhaps this should be replaced with `rand` crate; I did not took liberty of importing that as
-    // dependency.
-    struct Rng(u32);
-
-    impl Rng {
-        fn new() -> Self {
-            Rng(1234)
-        }
-
-        fn next(&mut self) -> u8 {
-            self.0 = (self.0 * 75 + 74) % (65537);
-
-            (self.0 % 256) as u8
-        }
-
-        fn next_vec(&mut self, len: usize) -> Vec<u8> {
-            let mut result = vec![];
-            for _ in 0..len {
-                result.push(self.next());
-            }
-
-            result
-        }
-    }
-
     #[cfg(feature = "unaligned")]
     #[test]
     fn test_verify_alignment_unaligned_mode() {
@@ -419,39 +398,36 @@ mod tests {
         assert_eq!(calculate_data_offset(101), Some(408));
     }
 
-    #[test]
-    fn test_no_alloc_buffer_segments_single_segment_optimization() {
-        let mut rng = Rng::new();
-        let segment_0 = rng.next_vec(128);
-        let output_segments = OutputSegments::SingleSegment([&segment_0]);
+    quickcheck! {
+        fn test_no_alloc_buffer_segments_single_segment_optimization(
+            segment_0 : Vec<Word>) -> TestResult
+        {
+            let words = &segment_0[..];
+            let bytes = Word::words_to_bytes(words);
+            let output_segments = OutputSegments::SingleSegment([bytes]);
+            let mut msg = vec![];
 
-        let mut msg = vec![];
+            serialize::write_message_segments(&mut msg, &output_segments).unwrap();
 
-        serialize::write_message_segments(&mut msg, &output_segments).unwrap();
+            let no_alloc_segments =
+                NoAllocSliceSegments::try_new(&mut msg.as_slice(), ReaderOptions::new()).unwrap();
 
-        let no_alloc_segments =
-            NoAllocSliceSegments::try_new(&mut msg.as_slice(), ReaderOptions::new()).unwrap();
+            assert!(matches!(
+                no_alloc_segments,
+                NoAllocSliceSegments::SingleSegment { segment } if segment == bytes
+            ));
 
-        assert!(matches!(
-            no_alloc_segments,
-            NoAllocSliceSegments::SingleSegment { segment } if segment == &segment_0[..]
-        ));
+            assert_eq!(no_alloc_segments.len(), 1);
+            assert_eq!(no_alloc_segments.get_segment(0), Some(bytes));
+            assert_eq!(no_alloc_segments.get_segment(1), None);
+            TestResult::from_bool(true)
+        }
 
-        assert_eq!(no_alloc_segments.len(), 1);
-        assert_eq!(no_alloc_segments.get_segment(0), Some(segment_0.as_slice()));
-        assert_eq!(no_alloc_segments.get_segment(1), None);
-    }
+        fn test_no_alloc_buffer_segments_multiple_segments(segments_vec: Vec<Vec<Word>>) -> TestResult {
+            if segments_vec.is_empty() { return TestResult::discard() };
 
-    #[test]
-    fn test_no_alloc_buffer_segments_multiple_segments() {
-        for count in 1..10 {
-            let mut rng = Rng::new();
-            let mut segments_vec = vec![];
-            for i in 0..count {
-                let vec_len = 8 * 2_usize.pow(i as u32);
-                segments_vec.push(rng.next_vec(vec_len));
-            }
-            let segments: Vec<_> = segments_vec.iter().map(|s| s.as_slice()).collect();
+            let segments: Vec<_> = segments_vec.iter().map(|s|
+                                                           Word::words_to_bytes(s.as_slice())).collect();
 
             let output_segments = OutputSegments::MultiSegment(segments.clone());
 
@@ -463,14 +439,15 @@ mod tests {
                 NoAllocSliceSegments::try_new(&mut msg.as_slice(), ReaderOptions::new()).unwrap();
 
             assert_eq!(no_alloc_segments.len(), segments.len());
-            for i in 0..no_alloc_segments.len() {
-                assert_eq!(no_alloc_segments.get_segment(i as u32), Some(segments[i]));
+            for (i, segment) in segments.iter().enumerate() {
+                assert_eq!(no_alloc_segments.get_segment(i as u32), Some(*segment));
             }
 
             assert_eq!(
                 no_alloc_segments.get_segment(no_alloc_segments.len() as u32),
                 None
             );
+            TestResult::from_bool(true)
         }
     }
 
@@ -511,77 +488,81 @@ mod tests {
         buf.clear();
     }
 
-    #[test]
-    fn test_no_alloc_buffer_segments_message_truncated() {
-        let mut rng = Rng::new();
-        let mut segments_vec = vec![];
-        for i in 0..5 {
-            let vec_len = 8 * 2_usize.pow(i as u32);
-            segments_vec.push(rng.next_vec(vec_len));
-        }
-        let segments: Vec<_> = segments_vec.iter().map(|s| s.as_slice()).collect();
+    quickcheck! {
+        fn test_no_alloc_buffer_segments_message_truncated(segments_vec: Vec<Vec<Word>>) -> TestResult {
+            if segments_vec.is_empty() { return TestResult::discard() }
 
-        let output_segments = OutputSegments::MultiSegment(segments.clone());
+            let segments: Vec<_> = segments_vec.iter()
+                .map(|s| Word::words_to_bytes(s.as_slice())).collect();
 
-        let mut msg = vec![];
+            let output_segments = OutputSegments::MultiSegment(segments.clone());
 
-        serialize::write_message_segments(&mut msg, &output_segments).unwrap();
-        msg.pop().unwrap();
+            let mut msg = vec![];
 
-        let no_alloc_segments =
-            NoAllocSliceSegments::try_new(&mut msg.as_slice(), ReaderOptions::new());
+            serialize::write_message_segments(&mut msg, &output_segments).unwrap();
 
-        assert!(no_alloc_segments.is_err());
-    }
+            // Lop off the final element.
+            msg.pop().unwrap();
 
-    #[test]
-    fn test_no_alloc_buffer_segments_message_options_limit() {
-        let mut rng = Rng::new();
-        let mut segments_vec = vec![];
-        for _ in 0..10 {
-            segments_vec.push(rng.next_vec(128));
-        }
-        let segments: Vec<_> = segments_vec.iter().map(|s| s.as_slice()).collect();
+            let no_alloc_segments =
+                NoAllocSliceSegments::try_new(&mut msg.as_slice(), ReaderOptions::new());
 
-        let output_segments = OutputSegments::MultiSegment(segments.clone());
-
-        let mut msg = vec![];
-
-        serialize::write_message_segments(&mut msg, &output_segments).unwrap();
-
-        let mut options = ReaderOptions::new();
-        options.traversal_limit_in_words(Some(1280 / 8));
-
-        let _no_alloc_segments =
-            NoAllocSliceSegments::try_new(&mut msg.as_slice(), options).unwrap();
-
-        let mut options = ReaderOptions::new();
-        options.traversal_limit_in_words(Some(1280 / 8 - 1));
-
-        let no_alloc_segments = NoAllocSliceSegments::try_new(&mut msg.as_slice(), options);
-
-        assert!(no_alloc_segments.is_err());
-    }
-
-    #[test]
-    fn test_no_alloc_buffer_segments_bad_alignment() {
-        let mut rng = Rng::new();
-        let segment_0 = rng.next_vec(128);
-        let output_segments = OutputSegments::SingleSegment([&segment_0]);
-
-        let mut msg = vec![];
-
-        serialize::write_message_segments(&mut msg, &output_segments).unwrap();
-        // mis-align buffer by 1 byte
-        msg.insert(0_usize, 0_u8);
-
-        let no_alloc_segments = NoAllocSliceSegments::try_new(&mut &msg[1..], ReaderOptions::new());
-
-        if cfg!(feature = "unaligned") {
-            // If we build with "unaligned" feature, alignment requirements should not be enforced
-            no_alloc_segments.unwrap();
-        } else {
             assert!(no_alloc_segments.is_err());
+            TestResult::from_bool(true)
+        }
+
+        fn test_no_alloc_buffer_segments_message_options_limit(
+            segments_vec: Vec<Vec<Word>>) -> TestResult
+        {
+            let mut word_count = 0;
+            let segments: Vec<_> = segments_vec.iter()
+                .map(|s| {
+                    let ws = Word::words_to_bytes(s.as_slice());
+                    word_count += s.len();
+                    ws
+                }).collect();
+            if word_count == 0 { return TestResult::discard() };
+
+            let output_segments = OutputSegments::MultiSegment(segments.clone());
+
+            let mut msg = vec![];
+
+            serialize::write_message_segments(&mut msg, &output_segments).unwrap();
+
+            let mut options = ReaderOptions::new();
+            options.traversal_limit_in_words(Some(word_count));
+
+            let _no_alloc_segments =
+                NoAllocSliceSegments::try_new(&mut msg.as_slice(), options).unwrap();
+
+            let mut options = ReaderOptions::new();
+            options.traversal_limit_in_words(Some(word_count - 1));
+
+            let no_alloc_segments = NoAllocSliceSegments::try_new(&mut msg.as_slice(), options);
+
+            assert!(no_alloc_segments.is_err());
+            TestResult::from_bool(true)
+        }
+
+        fn test_no_alloc_buffer_segments_bad_alignment(segment_0: Vec<Word>) -> TestResult {
+            if segment_0.is_empty() { return TestResult::discard(); }
+            let output_segments = OutputSegments::SingleSegment([Word::words_to_bytes(&segment_0)]);
+
+            let mut msg = vec![];
+
+            serialize::write_message_segments(&mut msg, &output_segments).unwrap();
+            // mis-align buffer by 1 byte
+            msg.insert(0_usize, 0_u8);
+
+            let no_alloc_segments = NoAllocSliceSegments::try_new(&mut &msg[1..], ReaderOptions::new());
+
+            if cfg!(feature = "unaligned") {
+                // If we build with "unaligned" feature, alignment requirements should not be enforced
+                no_alloc_segments.unwrap();
+            } else {
+                assert!(no_alloc_segments.is_err());
+            }
+            TestResult::from_bool(true)
         }
     }
 }
